#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import string
import subprocess

from sysrepo_interfaces import configObjNet


def config_wifi_interface(self, name, config, change_set):
    if not os.path.isdir('/sys/class/net/wifi0'):
        self.logger.info('No wifi device')
        return

    wifi_mode = self.get_interface_param(name, 'wifi-mode', 'client')
    if config:
        for key, value in config.items():
            if key == "wifi-mode":
                wifi_mode = value

    if change_set:
        for xpath, change in change_set.items():
            if xpath.find('wifi-mode') > -1:
                if change['operation'] == 'MODIFIED':
                    wifi_mode = change['new_value']

    self.logger.info("Interface {} is in {}".format(name, wifi_mode))
    if wifi_mode == 'client':
        self.execute_shell_command('systemctl --no-block --force stop wpa_supplicant 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed wpa_supplicant 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block disable hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block --force stop hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block --quiet stop wifi-auto 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed wifi-auto 2>/dev/null || true')
        with open('/etc/wpa_supplicant/wpa_supplicant.conf', 'w') as f:
            print('ctrl_interface=/var/run/wpa_supplicant', file=f)
            print('network={', file=f)
            print('ssid="dummydummy"', file=f)
            print('key_mgmt=NONE', file=f)
            print('}', file=f)

        config_wifi_client(self, name, config, change_set)
    elif wifi_mode == 'AP':
        self.execute_shell_command('systemctl --no-block disable wpa_supplicant 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block --force stop wpa_supplicant 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed wpa_supplicant 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block disable wifi-auto 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block --quiet stop wifi-auto 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed wifi-auto 2>/dev/null || true')
        config_wifi_ap(self, name, config, change_set)


def config_wifi_client(self, name, config, change_set):
    ssid = self.get_interface_param(name, "ssid")
    passphrase = self.get_interface_wifi_securty_param("passphrase")
    enable = self.get_interface_param(name, "enable")
    keymgmt = self.get_interface_wifi_securty_param("key-mgmt")
    country = self.get_interface_wifi_radio_param(name, "country")

    if config:
        for key, value in config.items():
            if key == "ssid":
                ssid = value
            if key == "security":
                if "passphrase" in config[key]:
                    passphrase = config[key]["passphrase"]
                if "key-mgmt" in config[key]:
                    keymgmt = config[key]["key-mgmt"]
            if key == "enable":
                enable = value
            if key == "radio-settings":
                if "country" in config[key]:
                    country = config[key]["country"]

    if change_set:
        for xpath, change in change_set.items():
            if change['operation'] == 'MODIFIED':
                if xpath.find('ssid') > -1:
                    ssid = change['new_value']
                if xpath.find('passphrase') > -1:
                    passphrase = change['new_value']
                if xpath.find('key-mgmt') > -1:
                    keymgmt = change['new_value']
                if xpath.find('enable') > -1 and "tracehost" not in xpath:
                    enable = change['new_value']
                if xpath.find('country') > -1:
                    country = change['new_value']

    if ssid is None or passphrase is None or enable is None:
        return

    try:
        if 'AUTO' == keymgmt:
            if not bool(change_set):
                # this is on boot, otherwise will call post_apply_config
                self.logger.info("wifi enabled on boot restart wifi-auto")
                self.execute_shell_command('systemctl --no-block enable wifi-auto 2>/dev/null || true')
                self.execute_shell_command('systemctl --no-block --quiet start wifi-auto 2>/dev/null || true')
        else:
            if 'WEP' != keymgmt:
                try:
                    wpa_out = subprocess.run(['/usr/bin/wpa_passphrase', ssid, passphrase],
                                             stdout=subprocess.PIPE, encoding='UTF-8')
                except subprocess.CalledProcessError as cpe:
                    self.logger.error('ERROR {}'.format(cpe))
                    pass
                except OSError:
                    self.logger.error('Missing /usr/bin/wpa_passphrase')
                    pass

                if 0 == wpa_out.returncode:
                    with open('/etc/wpa_supplicant/wpa_supplicant.conf', 'w') as f:
                        print('ctrl_interface=/run/wpa_supplicant', file=f)
                        print('update_config=1', file=f)
                        print('country={}'.format(country), file=f)
                        print('network={', file=f)
                        if 'WPA2-PSK' in keymgmt or 'CCMP' in keymgmt:
                            print('\tkey_mgmt={}'.format('WPA-PSK\n\tpairwise=CCMP'), file=f)
                        else:
                            print('\tkey_mgmt={}'.format('WPA-PSK\n\tpairwise=TKIP'), file=f)

                        print('{}'.format(wpa_out.stdout.split('\n', 1)[-1]), file=f)

            else:
                if passphrase:
                    passphrase_len = len(passphrase)
                    is_passphrase_in_hex = all(c in string.hexdigits for c in passphrase)
                    if True == is_passphrase_in_hex and passphrase_len not in (10, 26, 32):
                        self.logger.error('ERROR wrong WEP passphrase, setting 1234567890 !')
                        passphrase = str(1234567890)
                    elif False == is_passphrase_in_hex and passphrase_len not in (5, 13, 16):
                        self.logger.error('ERROR wrong WEP passphrase, setting 1234567890 !')
                        passphrase = str(1234567890)

                else:
                    print('ERROR empty WEP passphrase, setting 1234567890 !')
                    passphrase = str(1234567890)

                # check again
                is_passphrase_in_hex = all(c in string.hexdigits for c in passphrase)
                with open('/etc/wpa_supplicant/wpa_supplicant.conf', 'w') as f:
                    print('ctrl_interface=/run/wpa_supplicant', file=f)
                    print('update_config=1', file=f)
                    print('country={}'.format(country), file=f)
                    print('network={', file=f)
                    print('\tkey_mgmt={}'.format('NONE'), file=f)
                    print('\tssid="{}"'.format(ssid), file=f)
                    if is_passphrase_in_hex:
                        print('\twep_key0={}'.format(passphrase), file=f)
                    else:
                        print('\twep_key0={}'.format(int(hexlify(passphrase.encode()))), file=f)

                    print('}', file=f)

        if enable == 'true' or enable == True:
            if not bool(change_set):
                # this is on boot, otherwise will call post_apply_config
                self.logger.info('wifi enabled on boot restart wpa_supplicant')
                self.execute_shell_command('systemctl --no-block enable wpa_supplicant 2>/dev/null || true')
                self.execute_shell_command('systemctl --no-block --force stop wpa_supplicant 2>/dev/null || true')
                self.execute_shell_command('systemctl reset-failed wpa_supplicant 2>/dev/null || true')
                self.execute_shell_command('systemctl --no-block start wpa_supplicant 2>/dev/null || true')
                self.execute_shell_command('systemctl --no-block --force stop hostapd 2>/dev/null || true')
                self.execute_shell_command('systemctl reset-failed hostapd 2>/dev/null || true')
        else:
            self.execute_shell_command('systemctl --no-block disable wpa_supplicant 2>/dev/null || true')
            self.execute_shell_command('systemctl --no-block --force stop wpa_supplicant 2>/dev/null || true')

    except subprocess.CalledProcessError as ce:
        self.logger.error('FAIL wpa_supplicant {}'.format(ce))
    except subprocess.SubprocessError as se:
        self.logger.error('unknown error {}'.format(se))


def config_wifi_ap(self, name, config, change_set):
    ssid = self.get_interface_param(name, "ssid")
    passphrase = self.get_interface_wifi_securty_param("passphrase")
    enable = self.get_interface_param(name, "enable")
    keymgmt = self.get_interface_wifi_securty_param("key-mgmt")
    country = self.get_interface_wifi_radio_param(name, "country")
    band = self.get_interface_wifi_radio_param(name, "band")
    channel = self.get_interface_wifi_radio_param(name, "channel")
    tx_power = self.get_interface_wifi_radio_param(name, "transmit-power")
    hidden = self.get_interface_wifi_securty_param("hidden")
    isolation = self.get_interface_wifi_securty_param("isolation")
    max_clients = self.get_interface_wifi_securty_param("max-clients")
    mac_acl = self.get_interface_wifi_securty_param("mac-acl")
    mac_allow = self.get_mac_allow_list(name)
    mac_deny = self.get_mac_deny_list(name)
    cfg_update = False

    if config:
        for key, value in config.items():
            if key == "ssid":
                ssid = value

            if key == "security":
                if "passphrase" in config[key]:
                    passphrase = config[key]["passphrase"]
                if "key-mgmt" in config[key]:
                    keymgmt = config[key]["key-mgmt"]
                if "hidden" in config[key]:
                    hidden = config[key]["hidden"]
                if "isolation" in config[key]:
                    isolation = config[key]["isolation"]
                if "max-clients" in config[key]:
                    max_clients = config[key]["max-clients"]
                if "mac-acl" in config[key]:
                    mac_acl = config[key]["mac-acl"]
                if "mac-allow" in config[key]:
                    valuelist = config[key]["mac-allow"]
                    for item in valuelist:
                        mac_allow.append(item['mac'])
                if "mac-deny" in config[key]:
                    valuelist = config[key]["mac-deny"]
                    for item in valuelist:
                        mac_deny.append(item['mac'])

            if key == "enable":
                enable = value
                cfg_update = True

            if key == "radio-settings":
                if "country" in config[key]:
                    country = config[key]["country"]
                if "band" in config[key]:
                    band = config[key]["band"]
                if "channel" in config[key]:
                    channel = config[key]["channel"]
                if "transmit-power" in config[key]:
                    tx_power = config[key]["transmit-power"]

    if change_set:
        for xpath, change in change_set.items():
            if change['operation'] == 'DELETED':
                if xpath.find('mac-allow') > -1:
                    if not change['old_value']:
                        continue

                    self.logger.error('change allow mac delete {}'.format(change['old_value']))
                    if change['old_value'] in mac_allow:
                        mac_allow.remove(change['old_value'])

                    cfg_update = True
                if xpath.find('mac-deny') > -1:
                    if not change['old_value']:
                        continue

                    self.logger.error('change deny mac delete {}'.format(change['old_value']))
                    if change['old_value'] in mac_deny:
                        mac_deny.remove(change['old_value'])

                    cfg_update = True
            if change['operation'] == 'CREATED':
                if xpath.find('mac-allow') > -1:
                    if not change['new_value']:
                        continue
                    mac_allow.append(change['new_value'])
                    cfg_update = True
                if xpath.find('mac-deny') > -1:
                    if not change['new_value']:
                        continue
                    mac_deny.append(change['new_value'])
                    cfg_update = True

            if change['operation'] == 'MODIFIED':
                if xpath.find('ssid') > -1:
                    ssid = change['new_value']
                    cfg_update = True

                if xpath.find('passphrase') > -1:
                    passphrase = change['new_value']
                    cfg_update = True
                if xpath.find('key-mgmt') > -1:
                    keymgmt = change['new_value']
                    cfg_update = True
                if xpath.find('enable') > -1 and "tracehost" not in xpath:
                    enable = change['new_value']
                    cfg_update = True
                if xpath.find('country') > -1:
                    country = change['new_value']
                    cfg_update = True
                if xpath.find('band') > -1:
                    band = change['new_value']
                    cfg_update = True
                if xpath.find('channel') > -1:
                    channel = change['new_value']
                    cfg_update = True
                if xpath.find('transmit-power') > -1:
                    tx_power = change['new_value']
                    cfg_update = True
                if xpath.find('max-clients') > -1:
                    max_clients = change['new_value']
                    cfg_update = True
                if xpath.find('hidden') > -1:
                    hidden = change['new_value']
                    cfg_update = True
                if xpath.find('isolation') > -1:
                    isolation = change['new_value']
                    cfg_update = True
                if xpath.find('mac-acl') > -1:
                    mac_acl = change['new_value']
                    cfg_update = True

    cfg_obj = configObjNet.ConfigObjNet("/etc/hostapd/hostapd.conf")
    cfg_obj['interface'] = name
    cfg_obj['ieee80211d'] = 1
    cfg_obj['ieee80211h'] = 1
    if 'own_ip_addr' in cfg_obj:
        cfg_obj.pop('own_ip_addr')

    if ssid is not None:
        cfg_obj['ssid'] = ssid

    if passphrase is not None:
        if keymgmt is not None:
            if keymgmt == 'WEP':
                if 'wpa_passphrase' in cfg_obj:
                    cfg_obj.pop('wpa_passphrase')

                cfg_obj['wep_default_key'] = 0
                is_passphrase_in_hex = all(c in string.hexdigits for c in passphrase)
                passphrase_len = len(passphrase)
                if True == is_passphrase_in_hex and passphrase_len not in (10, 26, 32):
                    self.logger.error('ERROR: WEP password length {} != 10/26/32 !'.format(passphrase_len))
                    passphrase = str(1234567890)
                elif False == is_passphrase_in_hex and passphrase_len not in (5, 13, 16):
                    self.logger.error('ERROR: WEP password length {} != 5/13/16 !'.format(passphrase_len))
                    passphrase = str(1234567890)

                # check again
                is_passphrase_in_hex = all(c in string.hexdigits for c in passphrase)
                if True == is_passphrase_in_hex:
                    cfg_obj['wep_key0'] = passphrase
                else:
                    cfg_obj['wep_key0'] = int(hexlify(passphrase.encode()))
            else:
                if 'wep_default_key' in cfg_obj:
                    cfg_obj.pop('wep_default_key')

                if 'wep_key0' in cfg_obj:
                    cfg_obj.pop('wep_key0')

                cfg_obj['wpa_passphrase'] = passphrase
    if keymgmt is not None:
        cfg_obj['auth_algs'] = 1
        if keymgmt == 'WEP':
            if 'wpa_passphrase' in cfg_obj:
                cfg_obj.pop('wpa_passphrase')
            if 'wpa' in cfg_obj:
                cfg_obj.pop('wpa')
            if 'wpa_key_mgmt' in cfg_obj:
                cfg_obj.pop('wpa_key_mgmt')
            if 'wpa_pairwise' in cfg_obj:
                cfg_obj.pop('wpa_pairwise')
            if 'rsn_pairwise' in cfg_obj:
                cfg_obj.pop('rsn_pairwise')
        else:
            if 'wep_default_key' in cfg_obj:
                cfg_obj.pop('wep_default_key')
            if 'wep_key0' in cfg_obj:
                cfg_obj.pop('wep_key0')

        if keymgmt == 'WPA-PSK':
            cfg_obj['wpa'] = 1
            cfg_obj['wpa_key_mgmt'] = 'WPA-PSK'
            cfg_obj['wpa_pairwise'] = 'TKIP'
        if keymgmt == 'WPA2-PSK':
            cfg_obj['wpa'] = 2
            cfg_obj['wpa_key_mgmt'] = 'WPA-PSK'
            cfg_obj['rsn_pairwise'] = 'CCMP'

    if country is not None:
        cfg_obj['country_code'] = country

    if band is not None:
        if band == '5GHz':
            cfg_obj['hw_mode'] = 'a'
        else:
            cfg_obj['hw_mode'] = 'g'

        cfg_obj['ieee80211n'] = 1
        cfg_obj['wmm_enabled'] = 1

    if channel is not None:
        fiveg_channels = (32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56,
                          58, 60, 62, 64, 68, 96, 100, 102, 104, 106, 108, 110,
                          112, 114, 116, 118, 120, 122, 124, 126, 128,
                          132, 134, 136, 138, 140, 142, 144, 149,
                          151, 153, 155, 157, 159, 161, 165, 169, 173)
        try:
            hw_channels = subprocess.run(['/usr/bin/sudo', 'iw', 'phy', 'phy0', 'channels'],
                                         stdout=subprocess.PIPE, encoding='UTF-8')
        except subprocess.CalledProcessError as cpe:
            print('ERROR {}'.format(cpe))
            pass
        except OSError:
            print('Missing /usr/bin/sudo')
            pass

        available_2g_channels = list()
        available_5g_channels = list()
        if 0 == hw_channels.returncode:
            for ch in tuple(range(1, 15)):
                if '[{}] \n'.format(ch) in hw_channels.stdout:
                    available_2g_channels.append(ch)

            for ch in fiveg_channels:
                if '[{}] \n'.format(ch) in hw_channels.stdout:
                    available_5g_channels.append(ch)

        if '2.4GHz' == band and channel not in available_2g_channels:
            self.logger.error('ERROR: wrong channel {}, using {} !'.format(channel, available_2g_channels[0]))
            channel = available_2g_channels[0]
        elif '5GHz' == band and channel not in available_5g_channels:
            self.logger.error('ERROR: wrong channel {}, using {} !'.format(channel, available_5g_channels[0]))
            channel = available_5g_channels[0]

        cfg_obj['channel'] = channel

    if tx_power is not None:
        cfg_obj['local_pwr_constraint'] = tx_power

    if max_clients is not None:
        cfg_obj['max_num_sta'] = max_clients

    if hidden is not None:
        if hidden == 'true' or hidden is True:
            cfg_obj['ignore_broadcast_ssid'] = 1
        else:
            cfg_obj['ignore_broadcast_ssid'] = 0

    if isolation is not None:
        if isolation == 'true' or isolation is True:
            cfg_obj['ap_isolate'] = 1
        else:
            cfg_obj['ap_isolate'] = 0

    if mac_acl is not None:
        if mac_acl == 'allow':
            cfg_obj['macaddr_acl'] = 1
        if mac_acl == 'deny' or mac_acl == 'none':
            cfg_obj['macaddr_acl'] = 0

    cfg_obj['accept_mac_file'] = '/etc/hostapd/hostapd.accept'
    cfg_obj['deny_mac_file'] = '/etc/hostapd/hostapd.deny'

    self.write_file_to_disk(cfg_obj)

    with open('/etc/hostapd/hostapd.accept', 'w+') as f:
        mac_allow = map(lambda x: x + '\n', mac_allow)
        f.writelines(mac_allow)

    with open('/etc/hostapd/hostapd.deny', 'w+') as f:
        if mac_acl != 'none':
            mac_deny = map(lambda x: x + '\n', mac_deny)
            f.writelines(mac_deny)

    if enable == 'true' or enable == True:
        # if cfg_update:
        self.execute_shell_command('systemctl --no-block enable hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block --force stop hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block start hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block --force stop wifi-auto 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed wifi-auto 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block --force stop wpa_supplicant 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed wpa_supplicant 2>/dev/null || true')

    else:
        self.execute_shell_command('systemctl --no-block disable hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block --force stop hostapd 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed hostapd 2>/dev/null || true')

    # Configure and enable RA for IPv6
    ipv6_prefix = self.get_interface_param(name, 'ipv6-prefix', '')
    try:
        if ipv6_prefix:
            # radvd.add_ra(name, ipv6_prefix)
            self.radvd_ifs[name] = ipv6_prefix
            self.logger.debug(f"radvd configured for Wi-Fi: {name}")
        else:
            # radvd.del_ra(name)
            self.logger.debug(f"{name} doesn't have IPv6, dismissed for radvd config")
            if name in self.radvd_ifs.keys():
                del self.radvd_ifs[name]
    except:
        self.logger.error(f"Error occurred while configuring radvd for Wi-Fi interface: {name}")

    config_wifi_nat_interface(self, name, config, change_set)


def config_wifi_nat_interface(self, name, config, change_set):
    old_enable_nat = False
    if config and 'nat' in config:
        old_enable_nat = config['nat']

    if not config or 'wifi-wan-interface' not in config:
        config['wifi-wan-interface'] = []
    if change_set:
        for xpath, change in change_set.items():
            if xpath.find('enable') > -1 and "tracehost" not in xpath and change['operation'] != 'DELETED':
                config['nat'] = change['new_value']
                if 'true' in change['new_value'] or change['new_value'] is True:
                    config['nat'] = True
                elif 'false' in change['new_value']:
                    config['nat'] = False
            if xpath.find('wifi-wan-interface') > -1:
                if change['operation'] == 'DELETED':
                    if not change['old_value']:
                        continue
                    index = 0
                    for wan in config['wifi-wan-interface']:
                        if wan['name'] == change['old_value']:
                            wan_tag = self.get_interface_param(wan['name'], 'vlan', None)
                            self._clean_nat_iptables(name, wan['name'], wan_tag=wan_tag)
                            config['wifi-wan-interface'].pop(index)
                            if self.natpassthr is not None:
                                self.logger.debug('Delete wan interface \'{}\' from pass-thr for nat \'{}\' '.format(wan['name'], name))
                                self.natpassthr.nat_wan_del(name, wan['name'])
                            break
                        index = index + 1
                else:
                    if not change['new_value']:
                        continue
                    config['wifi-wan-interface'].append({'name': change['new_value']})

    if old_enable_nat:
        self._shell_return_ok("ifdown {}".format(name))
        self._delete_nat_interface(name, config)

    self._create_wifi_nat_interface(name, config, change_set)
    if 'nat' in config and config['nat']:
        self._shell_return_ok("ifup --ignore-error {}".format(name))

    if config and 'nat' in config and self.natpassthr is not None:
        self.logger.debug('Update nat interface state \'{}\' to pass-thr for nat \'{}\' '.format(config['nat'], name))
        self.natpassthr.nat_enable(name, config['nat'])


def post_apply_config_wifi(self, change_set):
    enable = self.get_interface_param('wifi0', 'enable')
    wifi_mode = self.get_interface_param('wifi0', 'wifi-mode')
    keymgmt = self.get_interface_wifi_securty_param('key-mgmt')

    try:
        if change_set:
            for change in change_set:
                if change['operation'] == 'MODIFIED':
                    if change['xpath'].find('wifi-mode') > -1:
                        wifi_mode = change['new_value']
                    if change['xpath'].find('enable') > -1:
                        enable = change['new_value']
                    if change['xpath'].find('key-mgmt') > -1:
                        keymgmt = change['new_value']
    except:
        pass

    if enable and (enable == 'true' or enable == True or enable == 'True') \
            and wifi_mode and wifi_mode == 'client':
        if keymgmt and keymgmt == 'AUTO':
            self.execute_shell_command('systemctl --no-block --force stop wifi-auto 2>/dev/null || true')
            self.execute_shell_command('systemctl reset-failed wifi-auto 2>/dev/null || true')
            self.execute_shell_command('systemctl --no-block --quiet start wifi-auto 2>/dev/null || true')
            self.logger.info("wifi enabled on change restart wifi-auto")

        self.logger.info("wifi enabled on change restart wpa_supplicant")
        self.execute_shell_command('systemctl --no-block --force stop wpa_supplicant 2>/dev/null || true')
        self.execute_shell_command('systemctl reset-failed wpa_supplicant 2>/dev/null || true')
        self.execute_shell_command('systemctl --no-block start wpa_supplicant 2>/dev/null || true')
        self._shell_return_ok('ifdown --ignore-error --force br-wifi0')
        self._shell_return_ok('ifup --ignore-error br-wifi0')
