import subprocess

def init(interfaces, logger=None):
   start_service = False
   with open("/etc/radvd.conf", 'w') as radvdconf:
      if interfaces:
         execute_shell_command("systemctl --no-block enable radvd.service")
         execute_shell_command("systemctl --no-block stop radvd.service")
         for interface, ipv6_prefix in interfaces.items():
            radvdconf.write("interface %s {\n" % (interface))
            radvdconf.write("        AdvSendAdvert on;\n")
            radvdconf.write("        AdvManagedFlag on;\n")
            radvdconf.write("        prefix %s {\n" % (ipv6_prefix))
            radvdconf.write("                AdvOnLink on;\n")
            radvdconf.write("                AdvAutonomous off;\n")
            radvdconf.write("        };\n")
            radvdconf.write("};\n\n")
         logger.info('Radvd initialized successfully')
         start_service = True
      else:
         radvdconf.truncate()
         execute_shell_command("systemctl --no-block stop radvd.service")
         execute_shell_command("systemctl --no-block disable radvd.service")
         logger.info('Radvd not needed. Configuration bypassed')
   if start_service:
       execute_shell_command('systemctl --no-block start radvd.service')


def execute_shell_command(cmd, shell=True):
   if cmd is str:
      cmd = str(cmd).split(' ')

   process = subprocess.Popen(
      cmd,
      stdout=subprocess.PIPE,
      stderr=subprocess.PIPE,
      shell=shell
   )
   (stdout, stderr) = process.communicate()
   if len(stderr) > 0 and process.returncode > 0:
      raise Exception("Error executing command {}!\n{}".format(cmd, stderr))
   return stdout.decode()


###########     OLD DESIGN     ###########
"""
LOCALHOST = 'localhost'

def init(logger=None):
   try:
      db = mysql.connector.connect(host=LOCALHOST, user="root", passwd="root", database="radvd")
   except:
      execute_shell_command("systemctl --no-block stop radvd.service")
      execute_shell_command("systemctl --no-block disable radvd.service")
      if logger:
         logger.error("Radvd error. Could not connect to database.")
         logger.error("Radvd service stopping...")
      return False
   cursor = db.cursor()
   
   execute_shell_command("systemctl --no-block enable radvd.service")
   execute_shell_command("systemctl --no-block stop radvd.service")
   
   cursor.execute("SELECT interface, ipv6, adv_managed_flag, adv_autonomous FROM radvd_config")
   all_data = cursor.fetchall()

   with open("/etc/radvd.conf", 'w') as radvdconf:
      for data in all_data:
         # 0 -> interface name
         # 1 -> IPv6 address
         # 2 -> AdvManagedFlag
         # 3 -> AdvAutonomous
         radvdconf.write("interface  %s {\n" % (data[0]))
         radvdconf.write("        AdvSendAdvert on;\n")
         if data[2] == False:
            radvdconf.write("        AdvManagedFlag off;\n")
         else:
            radvdconf.write("        AdvManagedFlag on;\n")
         radvdconf.write("        prefix %s {\n" % (data[1]))
         radvdconf.write("                AdvOnLink on;\n")
         if data[3] == True:
            radvdconf.write("                AdvAutonomous on;\n")
         else:
            radvdconf.write("                AdvAutonomous off;\n")
         radvdconf.write("        };\n")
         radvdconf.write("};\n")
   if not all_data:
      execute_shell_command("systemctl --no-block stop radvd.service")
      execute_shell_command("systemctl --no-block disable radvd.service")
   else:
      execute_shell_command('systemctl --no-block start radvd.service')
   return True


def __create_db():
   db = mysql.connector.connect(host=LOCALHOST, user="root", passwd="root")
   cursor = db.cursor()
   cursor.execute('CREATE DATABASE IF NOT EXISTS radvd')
   cursor.execute('USE radvd')
   cursor.execute('CREATE TABLE IF NOT EXISTS radvd_config (\
                     id INT AUTO_INCREMENT UNIQUE,\
                     interface VARCHAR(100) NOT NULL UNIQUE,\
                     ipv6 VARCHAR(50),\
                     adv_managed_flag BOOLEAN,\
                     adv_autonomous BOOLEAN,\
                     PRIMARY KEY (id)\
                  )')
   db.commit()
   db.close()


def add_ra(interface, ipv6, config=None, logger=None):
   try:
      __create_db()
      db = mysql.connector.connect(host=LOCALHOST, user="root", passwd="root", database="radvd")
   except:
      if logger:
         logger.error("Radvd error. Could not connect to database.")
      return   
   cursor = db.cursor()
   
   cursor.execute(f'SELECT interface FROM radvd_config WHERE interface="{str(interface)}"')
   interface_found = cursor.fetchall()

   # default for current config
   adv_managed_flag = '1'
   adv_autonomous = '0'
   
   # additional config parameters are created if there's a need in the future for statless IPv6 configuration
   if config:
      if config.find("adv_managed_flag") > -1:
         if config['adv_managed_flag'] == False:
            adv_managed_flag = '0'

      if config.find("adv_autonomous") > -1:
         if config['adv_autonomous'] == True:
            adv_autonomous = '1'

   # if there's already a configuration in radvd.conf for <interface>, update it, otherwise, insert new
   if interface_found:
      cursor.execute(f'UPDATE radvd_config SET ipv6="{ipv6}", adv_managed_flag="{adv_managed_flag}", adv_autonomous="{adv_autonomous}"  WHERE interface="{interface}"')
   else:
      cursor.execute(f'INSERT INTO radvd_config(interface, ipv6, adv_managed_flag, adv_autonomous) \
         VALUES ("{interface}", "{ipv6}", "{adv_managed_flag}", "{adv_autonomous}")')
   
   db.commit()
   db.close()


def del_ra(interface, logger=None):
   try:
      __create_db()
      db = mysql.connector.connect(host=LOCALHOST, user="root", passwd="root", database="radvd")
   except:
      if logger:
         logger.error("Radvd error. Could not connect to database.")
      return
   cursor = db.cursor()
   
   cursor.execute(f'SELECT interface FROM radvd_config WHERE interface="{str(interface)}"')
   data = cursor.fetchall()
   
   if data:
      cursor.execute(f'DELETE FROM radvd_config WHERE interface="{str(interface)}"')
   
   db.commit()
   db.close()
"""
